#include <stdio.h>
#include <string.h>
#include <maxminddb.h>
#include "IP2cntr.h"

static char *DatabasePath =
   "/var/db/maxmind/GeoLite2-Country/GeoLite2-Country.mmdb";

int IP2cntr (char *IPadress, char ISO_country[3], FILE *fperr)
{
   MMDB_s mmdb;
   int status, gai_error, mmdb_error, retval = 0;
   MMDB_lookup_result_s result;
   MMDB_entry_data_s entry_data;

   strcpy(ISO_country, "??");

   status = MMDB_open(DatabasePath, MMDB_MODE_MMAP, &mmdb);
   if (status != MMDB_SUCCESS)
   {
      if (fperr)
      {
         fprintf(fperr, "\nCould not open Country Database\n%s\n%s\n",
            DatabasePath, MMDB_strerror(status));
      }
      retval = 1;
      return retval;
   }
   result = MMDB_lookup_string(&mmdb, IPadress, &gai_error, &mmdb_error);
   if (gai_error != 0)
   {
      if (fperr)
      {
         fprintf(fperr, "\nGai-error looking up IP\n%s\n", MMDB_strerror(gai_error));
      }
      retval = 2;
      goto close_and_end;
   }
   if (mmdb_error != MMDB_SUCCESS)
   {
      if (fperr)
      {
         fprintf(fperr, "\nMmdb-error looking up IP address\n%s\n", MMDB_strerror(mmdb_error));
      }
      retval = 3;
      goto close_and_end;
   }
   if (!result.found_entry)
   {
      if (fperr)
      {
         fprintf(fperr, "\nIP adress not found\n%s\n", MMDB_strerror(mmdb_error));
      }
      retval = 4;
      goto close_and_end;
   }
   else
   {
      status = MMDB_get_value(&result.entry, &entry_data, "country", "iso_code", NULL);
      if (status != MMDB_SUCCESS)
      {
         if (fperr)
         {
            fprintf(fperr, "\nCould not get value\n%s\n", MMDB_strerror(status));
         }
         retval = 5;
         goto close_and_end;
      }
      if (entry_data.has_data && entry_data.type == MMDB_DATA_TYPE_UTF8_STRING)
      {
         sprintf(ISO_country, "%.*s",
            entry_data.data_size <= 2 ? entry_data.data_size : 2,
            entry_data.utf8_string);
      }
   }

   close_and_end: MMDB_close(&mmdb);

   return retval;
}
